@@ -314,9 +314,12 @@ class Module
         $navigation = new Navigation();
         foreach ($panes as $pane) {
             /** @var DashboardContainer $pane */
-            $dashlets = array();
+            $dashlets = [];
             foreach ($pane->getDashlets() as $dashletName => $dashletUrl) {
-                $dashlets[$this->translate($dashletName)] = $dashletUrl;
+                $dashlets[$dashletName] = [
+                    'label' => $this->translate($dashletName),
+                    'url'   => $dashletUrl
+                ];
             }
 
             $navigation->addItem(
@@ -326,7 +329,7 @@ class Module
                     array(
                         'label'     => $this->translate($pane->getName()),
                         'type'      => 'dashboard-pane',
-                        'dashlets'  => $dashlets
+                        'children'  => $dashlets
                     )
                 )
             );
@@ -362,25 +365,28 @@ class Module
     public function getMenu()
     {
         $this->launchConfigScript();
-        return $this->createMenu($this->menuItems);
+        return Navigation::fromArray($this->createMenu($this->menuItems));
     }
 
     /**
-     * Create and return a new navigation for the given menu items
+     * Create and return an array structure for the given menu items
      *
      * @param   MenuItemContainer[]     $items
      *
-     * @return  Navigation
+     * @return  array
      */
     private function createMenu(array $items)
     {
-        $navigation = new Navigation();
+        $navigation = array();
         foreach ($items as $item) {
             /** @var MenuItemContainer $item */
-            $navigationItem = $navigation->createItem($item->getName(), $item->getProperties());
-            $navigationItem->setChildren($this->createMenu($item->getChildren()));
-            $navigationItem->setLabel($this->translate($item->getName()));
-            $navigation->addItem($navigationItem);
+            $properties = $item->getProperties();
+            $properties['children'] = $this->createMenu($item->getChildren());
+            if (! isset($properties['label'])) {
+                $properties['label'] = $this->translate($item->getName());
+            }
+
+            $navigation[$item->getName()] = $properties;
         }
 
         return $navigation;
@@ -525,7 +531,9 @@ class Module
     {
         $this->launchConfigScript();
         $files = $this->cssFiles;
-        $files[] = $this->getCssFilename();
+        if (file_exists($this->getCssFilename())) {
+            $files[] = $this->getCssFilename();
+        }
         return $files;
     }
 
@@ -657,7 +665,6 @@ class Module
             );
 
             if (file_exists($this->metadataFile)) {
-
                 $key = null;
                 $file = new File($this->metadataFile, 'r');
                 foreach ($file as $line) {
@@ -679,11 +686,10 @@ class Module
                     $key = lcfirst($key);
 
                     switch ($key) {
-
                         case 'depends':
                             if (strpos($val, ' ') === false) {
                                 $metadata->depends[$val] = true;
-                                continue;
+                                continue 2;
                             }
 
                             $parts = preg_split('/,\s+/', $val);
@@ -707,7 +713,6 @@ class Module
 
                         default:
                             $metadata->{$key} = $val;
-
                     }
                 }
             }
@@ -874,9 +879,13 @@ class Module
             'urlParams' => array('name' => $this->getName()),
             'label'     => 'Module: ' . $this->getName()
         ));
-        foreach ($this->configTabs as $name => $config) {
-            $tabs->add($name, $config);
+
+        if ($this->app->getModuleManager()->hasEnabled($this->name)) {
+            foreach ($this->configTabs as $name => $config) {
+                $tabs->add($name, $config);
+            }
         }
+
         return $tabs;
     }
 
@@ -1248,7 +1257,7 @@ class Module
      *
      * @return  $this
      *
-     * @deprecated              Deprecated in favor of {@link provideHook()}. Will be removed in version 2.2.0
+     * @deprecated              Deprecated since 2.1.1. Use {@link provideHook()} instead
      */
     protected function registerHook($name, $class, $key = null)
     {
@@ -1269,15 +1278,14 @@ class Module
      * Provide a hook implementation
      *
      * @param   string  $name           Name of the hook for which to provide an implementation
-     * @param   string  $implementation [optional] Fully qualified name of the class providing the hook implementation.
+     * @param   string  $implementation Fully qualified name of the class providing the hook implementation.
      *                                  Defaults to the module's ProvidedHook namespace plus the hook's name for the
-     *                                  class name. Web 2's namespace separator is \\ (double backslash) at the moment
-     * @param   string  $key            No-op arg for compatibility reasons. This argument is deprecated and will be
-     *                                  removed in version 2.2.0
+     *                                  class name
+     * @param   bool    $alwaysRun      To run the hook always (e.g. without permission check)
      *
      * @return  $this
      */
-    protected function provideHook($name, $implementation = null, $key = null)
+    protected function provideHook($name, $implementation = null, $alwaysRun = false)
     {
         if ($implementation === null) {
             $implementation = $name;
@@ -1291,7 +1299,7 @@ class Module
             $class = $implementation;
         }
 
-        Hook::register($name, $class, $class);
+        Hook::register($name, $class, $class, $alwaysRun);
         return $this;
     }
 
