@@ -3,6 +3,8 @@
 
 namespace Icinga\Data\Db;
 
+use DateTime;
+use DateTimeZone;
 use Exception;
 use Icinga\Data\Inspectable;
 use Icinga\Data\Inspection;
@@ -185,6 +187,11 @@ class DbConnection implements Selectable, Extensible, Updatable, Reducible, Insp
                     if ($this->config->ssl_cipher) {
                         $adapterParamaters['driver_options'][PDO::MYSQL_ATTR_SSL_CIPHER] = $this->config->ssl_cipher;
                     }
+                    if (defined('PDO::MYSQL_ATTR_SSL_VERIFY_SERVER_CERT')
+                        && $this->config->ssl_do_not_verify_server_cert
+                    ) {
+                        $adapterParamaters['driver_options'][PDO::MYSQL_ATTR_SSL_VERIFY_SERVER_CERT] = false;
+                    }
                 }
                 /*
                  * Set MySQL server SQL modes to behave as closely as possible to Oracle and PostgreSQL. Note that the
@@ -195,11 +202,12 @@ class DbConnection implements Selectable, Extensible, Updatable, Reducible, Insp
                  */
                 $driverOptions[PDO::MYSQL_ATTR_INIT_COMMAND] =
                     'SET SESSION SQL_MODE=\'STRICT_ALL_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,'
-                    . 'NO_AUTO_CREATE_USER,ANSI_QUOTES,PIPES_AS_CONCAT,NO_ENGINE_SUBSTITUTION\'';
+                    . 'ANSI_QUOTES,PIPES_AS_CONCAT,NO_ENGINE_SUBSTITUTION\'';
                 if (isset($adapterParamaters['charset'])) {
                     $driverOptions[PDO::MYSQL_ATTR_INIT_COMMAND] .= ', NAMES ' . $adapterParamaters['charset'];
                     unset($adapterParamaters['charset']);
                 }
+                $driverOptions[PDO::MYSQL_ATTR_INIT_COMMAND] .= ", time_zone='" . $this->defaultTimezoneOffset() . "'";
                 $driverOptions[PDO::MYSQL_ATTR_INIT_COMMAND] .=';';
                 $defaultPort = 3306;
                 break;
@@ -215,6 +223,11 @@ class DbConnection implements Selectable, Extensible, Updatable, Reducible, Insp
             case 'oracle':
                 $adapter = 'Pdo_Oci';
                 $defaultPort = 1521;
+
+                // remove host parameter when not configured
+                if (empty($this->config->host)) {
+                    unset($adapterParamaters['host']);
+                }
                 break;
             case 'pgsql':
                 $adapter = 'Pdo_Pgsql';
@@ -277,6 +290,22 @@ class DbConnection implements Selectable, Extensible, Updatable, Reducible, Insp
         return $this;
     }
 
+    /**
+     * Get offset from the current default timezone to GMT
+     *
+     * @return string
+     */
+    protected function defaultTimezoneOffset()
+    {
+        $tz = new DateTimeZone(date_default_timezone_get());
+        $offset = $tz->getOffset(new DateTime());
+        $prefix = $offset >= 0 ? '+' : '-';
+        $offset = abs($offset);
+        $hours = (int) floor($offset / 3600);
+        $minutes = (int) floor(($offset % 3600) / 60);
+        return sprintf('%s%d:%02d', $prefix, $hours, $minutes);
+    }
+
     /**
      * Count all rows of the result set
      *
@@ -555,7 +584,7 @@ class DbConnection implements Selectable, Extensible, Updatable, Reducible, Insp
                 'Connection to %s as %s on %s:%s successful',
                 $config['dbname'],
                 $config['username'],
-                $config['host'],
+                array_key_exists('host', $config) ? $config['host'] : '(none)',
                 $config['port']
             ));
             switch ($this->dbType) {
