@@ -11,6 +11,10 @@ $this->providePermission(
     'monitoring/command/schedule-check',
     $this->translate('Allow scheduling host and service checks')
 );
+$this->providePermission(
+    'monitoring/command/schedule-check/active-only',
+    $this->translate('Allow scheduling host and service checks (Only on objects with active checks enabled)')
+);
 $this->providePermission(
     'monitoring/command/acknowledge-problem',
     $this->translate('Allow acknowledging host and service problems')
@@ -104,6 +108,7 @@ $this->provideSetupWizard('Icinga\Module\Monitoring\MonitoringWizard');
 /*
  * Available Search Urls
  */
+$this->provideSearchUrl($this->translate('Tactical Overview'), 'monitoring/tactical', 100);
 $this->provideSearchUrl($this->translate('Hosts'), 'monitoring/list/hosts?sort=host_severity&limit=10', 99);
 $this->provideSearchUrl($this->translate('Services'), 'monitoring/list/services?sort=service_severity&limit=10', 98);
 $this->provideSearchUrl($this->translate('Hostgroups'), 'monitoring/list/hostgroups?limit=10', 97);
@@ -126,108 +131,142 @@ $section = $this->menuSection(N_('Problems'), array(
         'SummaryNavigationItemRenderer',
         'state' => 'critical'
     ),
-    'icon'      => 'block',
+    'icon'      => 'attention-circled',
     'priority'  => 20
 ));
 $section->add(N_('Host Problems'), array(
-    'renderer'  => array(
+    'icon'        => 'host',
+    'description' => $this->translate('List current host problems'),
+    'renderer'    => array(
         'MonitoringBadgeNavigationItemRenderer',
         'columns' => array(
             'hosts_down_unhandled' => $this->translate('%d unhandled hosts down')
         ),
         'state'    => 'critical',
-        'dataView' => 'statussummary'
+        'dataView' => 'unhandledhostproblems'
     ),
     'url'       => 'monitoring/list/hosts?host_problem=1&sort=host_severity',
     'priority'  => 50
 ));
 $section->add(N_('Service Problems'), array(
-    'renderer'  => array(
+    'icon'        => 'service',
+    'description' => $this->translate('List current service problems'),
+    'renderer'    => array(
         'MonitoringBadgeNavigationItemRenderer',
         'columns' => array(
             'services_critical_unhandled' => $this->translate('%d unhandled services critical')
         ),
         'state'    => 'critical',
-        'dataView' => 'statussummary'
+        'dataView' => 'unhandledserviceproblems'
     ),
     'url'       => 'monitoring/list/services?service_problem=1&sort=service_severity&dir=desc',
     'priority'  => 60
 ));
 $section->add(N_('Service Grid'), array(
-    'url'       => 'monitoring/list/servicegrid?problems',
-    'priority'  => 70
+    'icon'        => 'services',
+    'description' => $this->translate('Display service problems as grid'),
+    'url'         => 'monitoring/list/servicegrid?problems',
+    'priority'    => 70
 ));
 $section->add(N_('Current Downtimes'), array(
-    'url'       => 'monitoring/list/downtimes?downtime_is_in_effect=1',
-    'priority'  => 80
+    'icon'        => 'plug',
+    'description' => $this->translate('List current downtimes'),
+    'url'         => 'monitoring/list/downtimes?downtime_is_in_effect=1',
+    'priority'    => 80
 ));
 
 /*
  * Overview Section
  */
 $section = $this->menuSection(N_('Overview'), array(
-    'icon'      => 'sitemap',
+    'icon'      => 'binoculars',
     'priority'  => 30
 ));
 $section->add(N_('Tactical Overview'), array(
-    'url'      => 'monitoring/tactical',
-    'priority' => 40
+    'icon'        => 'chart-pie',
+    'description' => $this->translate('Open tactical overview'),
+    'url'         => 'monitoring/tactical',
+    'priority'    => 40
 ));
 $section->add(N_('Hosts'), array(
-    'url'      => 'monitoring/list/hosts',
-    'priority' => 50
+    'icon'        => 'host',
+    'description' => $this->translate('List hosts'),
+    'url'         => 'monitoring/list/hosts',
+    'priority'    => 50
 ));
 $section->add(N_('Services'), array(
-    'url'      => 'monitoring/list/services',
-    'priority' => 50
+    'icon'        => 'service',
+    'description' => $this->translate('List services'),
+    'url'         => 'monitoring/list/services',
+    'priority'    => 50
 ));
 $section->add(N_('Servicegroups'), array(
-    'url'      => 'monitoring/list/servicegroups',
-    'priority' => 60
+    'icon'        => 'services',
+    'description' => $this->translate('List service groups'),
+    'url'         => 'monitoring/list/servicegroups',
+    'priority'    => 60
 ));
 $section->add(N_('Hostgroups'), array(
-    'url'      => 'monitoring/list/hostgroups',
-    'priority' => 60
+    'icon'        => 'host',
+    'description' => $this->translate('List host groups'),
+    'url'         => 'monitoring/list/hostgroups',
+    'priority'    => 60
 ));
 $section->add(N_('Contacts'), array(
-    'url'      => 'monitoring/list/contacts',
-    'priority' => 70
+    'icon'        => 'user',
+    'description' => $this->translate('List contacts'),
+    'url'         => 'monitoring/list/contacts',
+    'priority'    => 70
 ));
 $section->add(N_('Contactgroups'), array(
-    'url'      => 'monitoring/list/contactgroups',
-    'priority' => 70
+    'icon'        => 'users',
+    'description' => $this->translate('List users'),
+    'url'         => 'monitoring/list/contactgroups',
+    'priority'    => 70
 ));
 $section->add(N_('Comments'), array(
-    'url'      => 'monitoring/list/comments?comment_type=(comment|ack)',
-    'priority' => 80
+    'icon'        => 'chat-empty',
+    'description' => $this->translate('List comments'),
+    'url'         => 'monitoring/list/comments?comment_type=comment|comment_type=ack',
+    'priority'    => 80
 ));
 $section->add(N_('Downtimes'), array(
-    'url'      => 'monitoring/list/downtimes',
-    'priority' => 80
+    'icon'        => 'plug',
+    'description' => $this->translate('List downtimes'),
+    'url'         => 'monitoring/list/downtimes',
+    'priority'    => 80
 ));
 
 /*
  * History Section
  */
 $section = $this->menuSection(N_('History'), array(
-    'icon'      => 'rewind',
+    'icon'      => 'history',
     'priority'  => 90
 ));
 $section->add(N_('Event Grid'), array(
-    'priority'  => 10,
-    'url'       => 'monitoring/list/eventgrid'
+    'icon'        => 'history',
+    'description' => $this->translate('Open event grid'),
+    'priority'    => 10,
+    'url'         => 'monitoring/list/eventgrid'
 ));
 $section->add(N_('Event Overview'), array(
-    'priority'  => 20,
-    'url'       => 'monitoring/list/eventhistory?timestamp>=-7%20days'
+    'icon'        => 'history',
+    'description' => $this->translate('Open event overview'),
+    'priority'    => 20,
+    'url'         => 'monitoring/list/eventhistory?timestamp>=-7%20days'
 ));
 $section->add(N_('Notifications'), array(
-    'priority'  => 30,
-    'url'       => 'monitoring/list/notifications',
+    'icon'        => 'bell',
+    'description' => $this->translate('List notifications'),
+    'priority'    => 30,
+    'url'         => 'monitoring/list/notifications?notification_timestamp>=-7%20days',
 ));
 $section->add(N_('Timeline'), array(
-    'priority'  => 40,
-    'url'       => 'monitoring/timeline'
+    'icon'        => 'clock',
+    'description' => $this->translate('Open timeline'),
+    'priority'    => 40,
+    'url'         => 'monitoring/timeline'
 ));
 
 /*
@@ -238,18 +277,16 @@ $section = $this->menuSection(N_('Reporting'), array(
     'priority'  => 100
 ));
 
-$section->add(N_('Alert Summary'), array(
-   'url'    => 'monitoring/alertsummary/index'
-));
-
 /*
  * System Section
  */
 $section = $this->menuSection(N_('System'));
 $section->add(N_('Monitoring Health'), array(
-    'url'      => 'monitoring/health/info',
-    'priority' => 720,
-    'renderer' => 'BackendAvailabilityNavigationItemRenderer'
+    'icon'        => 'check',
+    'description' => $this->translate('Open monitoring health'),
+    'url'         => 'monitoring/health/info',
+    'priority'    => 720,
+    'renderer'    => 'BackendAvailabilityNavigationItemRenderer'
 ));
 
 /*
