@@ -3,8 +3,6 @@
 
 namespace Icinga\Module\Monitoring\Backend\Ido\Query;
 
-use Icinga\Exception\NotImplementedError;
-
 class ServicegroupQuery extends IdoQuery
 {
     protected $groupBase = array(
@@ -22,6 +20,18 @@ class ServicegroupQuery extends IdoQuery
     );
 
     protected $columnMap = array(
+        'contacts' => [
+            'service_contact' => 'sco.name1'
+        ],
+        'contactgroups' => [
+            'service_contactgroup' => 'scgo.name1'
+        ],
+        'hostcontacts' => [
+            'host_contact' => 'hco.name1'
+        ],
+        'hostcontactgroups' => [
+            'host_contactgroup' => 'hcgo.name1'
+        ],
         'hostgroups' => array(
             'hostgroup_name' => 'hgo.name1'
         ),
@@ -96,6 +106,86 @@ class ServicegroupQuery extends IdoQuery
         $this->joinedVirtualTables = array('servicegroups' => true);
     }
 
+    /**
+     * Join contacts
+     */
+    protected function joinContacts()
+    {
+        $this->requireVirtualTable('services');
+
+        $this->select->joinLeft(
+            ['sc' => 'icinga_service_contacts'],
+            'sc.service_id = s.service_id',
+            []
+        )->joinLeft(
+            ['sco' => 'icinga_objects'],
+            'sco.object_id = sc.contact_object_id AND sco.is_active = 1 AND sco.objecttype_id = 10',
+            []
+        );
+    }
+
+    /**
+     * Join contact groups
+     */
+    protected function joinContactgroups()
+    {
+        $this->requireVirtualTable('services');
+
+        $this->select->joinLeft(
+            ['scg' => 'icinga_service_contactgroups'],
+            'scg.service_id = s.service_id',
+            []
+        )->joinLeft(
+            ['scgo' => 'icinga_objects'],
+            'scgo.object_id = scg.contactgroup_object_id AND scgo.is_active = 1 AND scgo.objecttype_id = 10',
+            []
+        );
+    }
+
+    /**
+     * Join host contacts
+     */
+    protected function joinHostcontacts()
+    {
+        $this->requireVirtualTable('services');
+
+        $this->select->joinLeft(
+            ['h' => 'icinga_hosts'],
+            'h.host_object_id = s.host_object_id',
+            []
+        )->joinLeft(
+            ['hc' => 'icinga_host_contacts'],
+            'hc.host_id = h.host_id',
+            []
+        )->joinLeft(
+            ['hco' => 'icinga_objects'],
+            'hco.object_id = hc.contact_object_id AND hco.is_active = 1 AND hco.objecttype_id = 10',
+            []
+        );
+    }
+
+    /**
+     * Join host contact groups
+     */
+    protected function joinHostcontactgroups()
+    {
+        $this->requireVirtualTable('services');
+
+        $this->select->joinLeft(
+            ['h' => 'icinga_hosts'],
+            'h.host_object_id = s.host_object_id',
+            []
+        )->joinLeft(
+            ['hcg' => 'icinga_host_contactgroups'],
+            'hcg.host_id = h.host_id',
+            []
+        )->joinLeft(
+            ['hcgo' => 'icinga_objects'],
+            'hcgo.object_id = hcg.contactgroup_object_id AND hcgo.is_active = 1 AND hcgo.objecttype_id = 11',
+            []
+        );
+    }
+
     /**
      * Join host groups
      */
@@ -125,6 +215,9 @@ class ServicegroupQuery extends IdoQuery
     protected function joinHosts()
     {
         $this->requireVirtualTable('services');
+
+        // Host custom var filters work w/o any host related table. If a host table join is necessary here some day,
+        // please adjust `joinHostcontact*()` where we explicitly do this already
     }
 
     /**
@@ -195,13 +288,10 @@ class ServicegroupQuery extends IdoQuery
 
             return ['so.object_id', 'so.object_id'];
         } elseif ($name === 'servicegroup') {
-            if (! $and) {
-                // IN AND NOT IN for OR filters works w/o subquery joins
-                throw new NotImplementedError('');
-            } else {
-                // Propagate that the "parent" query has to be filtered as well
-                $additionalFilter = clone $filter;
-            }
+            // Propagate that the "parent" query has to be filtered as well
+            $additionalFilter = clone $filter;
+
+            $this->requireVirtualTable('members');
 
             $query->joinVirtualTable('members');
 
