@@ -23,6 +23,11 @@ class ServicestatusQuery extends IdoQuery
      */
     protected $groupOrigin = array('hostgroups', 'servicegroups');
 
+    protected $subQueryTargets = array(
+        'hostgroups'    => 'hostgroup',
+        'servicegroups' => 'servicegroup'
+    );
+
     /**
      * {@inheritdoc}
      */
@@ -30,6 +35,18 @@ class ServicestatusQuery extends IdoQuery
         'checktimeperiods' => array(
             'service_check_timeperiod' => 'ctp.alias COLLATE latin1_general_ci'
         ),
+        'contacts' => [
+            'service_contact' => 'sco.name1'
+        ],
+        'contactgroups' => [
+            'service_contactgroup' => 'scgo.name1'
+        ],
+        'hostcontacts' => [
+            'host_contact' => 'hco.name1'
+        ],
+        'hostcontactgroups' => [
+            'host_contactgroup' => 'hcgo.name1'
+        ],
         'hostgroups' => array(
             'hostgroup'         => 'hgo.name1 COLLATE latin1_general_ci',
             'hostgroup_alias'   => 'hg.alias COLLATE latin1_general_ci',
@@ -138,6 +155,7 @@ class ServicestatusQuery extends IdoQuery
             'object_type'               => '(\'service\')',
             'service'                   => 'so.name2 COLLATE latin1_general_ci',
             'service_action_url'        => 's.action_url',
+            'service_check_interval'    => '(s.check_interval * 60)',
             'service_description'       => 'so.name2',
             'service_display_name'      => 's.display_name COLLATE latin1_general_ci',
             'service_host'              => 'so.name1 COLLATE latin1_general_ci',
@@ -198,14 +216,14 @@ class ServicestatusQuery extends IdoQuery
                 NULL
             ELSE
                 UNIX_TIMESTAMP(ss.last_check)
-                + CASE WHEN
+                + (CASE WHEN
                     COALESCE(ss.current_state, 0) > 0 AND ss.state_type = 0
                 THEN
                     ss.retry_check_interval
                 ELSE
                     ss.normal_check_interval
-                END * 60 * 2
-                + CEIL(ss.execution_time)
+                END * 60
+                + CEIL(ss.execution_time) + CEIL(ss.latency)) * 2
             END',
             'service_no_more_notifications'             => 'ss.no_more_notifications',
             'service_normal_check_interval'             => 'ss.normal_check_interval',
@@ -262,7 +280,8 @@ class ServicestatusQuery extends IdoQuery
             'service_state'                             => 'CASE WHEN ss.has_been_checked = 0 OR ss.has_been_checked IS NULL THEN 99 ELSE ss.current_state END',
             'service_state_type'                        => 'ss.state_type',
             'service_status_update_time'                => 'ss.status_update_time',
-            'service_unhandled'                         => 'CASE WHEN (ss.problem_has_been_acknowledged + ss.scheduled_downtime_depth + COALESCE(hs.current_state, 0)) = 0 THEN 1 ELSE 0 END'
+            'service_unhandled'                         => 'CASE WHEN (ss.problem_has_been_acknowledged + ss.scheduled_downtime_depth + COALESCE(hs.current_state, 0)) = 0 THEN 1 ELSE 0 END',
+            'problems'                                  => 'CASE WHEN COALESCE(ss.current_state, 0) = 0 THEN 0 ELSE 1 END'
         )
     );
 
@@ -303,6 +322,74 @@ class ServicestatusQuery extends IdoQuery
         );
     }
 
+    /**
+     * Join contacts
+     */
+    protected function joinContacts()
+    {
+        $this->select->joinLeft(
+            ['sc' => 'icinga_service_contacts'],
+            'sc.service_id = s.service_id',
+            []
+        )->joinLeft(
+            ['sco' => 'icinga_objects'],
+            'sco.object_id = sc.contact_object_id AND sco.is_active = 1 AND sco.objecttype_id = 10',
+            []
+        );
+    }
+
+    /**
+     * Join contact groups
+     */
+    protected function joinContactgroups()
+    {
+        $this->select->joinLeft(
+            ['scg' => 'icinga_service_contactgroups'],
+            'scg.service_id = s.service_id',
+            []
+        )->joinLeft(
+            ['scgo' => 'icinga_objects'],
+            'scgo.object_id = scg.contactgroup_object_id AND scgo.is_active = 1 AND scgo.objecttype_id = 10',
+            []
+        );
+    }
+
+    /**
+     * Join host contacts
+     */
+    protected function joinHostcontacts()
+    {
+        $this->requireVirtualTable('hosts');
+
+        $this->select->joinLeft(
+            ['hc' => 'icinga_host_contacts'],
+            'hc.host_id = h.host_id',
+            []
+        )->joinLeft(
+            ['hco' => 'icinga_objects'],
+            'hco.object_id = hc.contact_object_id AND hco.is_active = 1 AND hco.objecttype_id = 10',
+            []
+        );
+    }
+
+    /**
+     * Join host contact groups
+     */
+    protected function joinHostcontactgroups()
+    {
+        $this->requireVirtualTable('hosts');
+
+        $this->select->joinLeft(
+            ['hcg' => 'icinga_host_contactgroups'],
+            'hcg.host_id = h.host_id',
+            []
+        )->joinLeft(
+            ['hcgo' => 'icinga_objects'],
+            'hcgo.object_id = hcg.contactgroup_object_id AND hcgo.is_active = 1 AND hcgo.objecttype_id = 11',
+            []
+        );
+    }
+
     /**
      * Join host groups
      */
@@ -411,4 +498,19 @@ class ServicestatusQuery extends IdoQuery
             parent::registerGroupColumns($alias, $table, $groupedColumns, $groupedTables);
         }
     }
+
+    protected function joinSubQuery(IdoQuery $query, $name, $filter, $and, $negate, &$additionalFilter)
+    {
+        if ($name === 'hostgroup') {
+            $query->joinVirtualTable('members');
+
+            return ['hgm.host_object_id', 's.host_object_id'];
+        } elseif ($name === 'servicegroup') {
+            $query->joinVirtualTable('members');
+
+            return ['sgm.service_object_id', 'so.object_id'];
+        }
+
+        return parent::joinSubQuery($query, $name, $filter, $and, $negate, $additionalFilter);
+    }
 }
