@@ -17,6 +17,7 @@ class Servicegroupsummary extends DataView
             'services_critical_unhandled',
             'services_ok',
             'services_pending',
+            'services_severity',
             'services_total',
             'services_unknown_handled',
             'services_unknown_unhandled',
@@ -35,6 +36,13 @@ class Servicegroupsummary extends DataView
         return array(
             'servicegroup_alias' => array(
                 'order' => self::SORT_ASC
+            ),
+            'services_severity' => array(
+                'columns' => array(
+                    'services_severity',
+                    'servicegroup_alias ASC'
+                ),
+                'order' => self::SORT_DESC
             )
         );
     }
@@ -44,9 +52,9 @@ class Servicegroupsummary extends DataView
         return array(
             'instance_name',
             'services_severity',
-            'host', 'host_alias', 'host_display_name', 'host_name',
+            'host', 'host_alias', 'host_contact', 'host_contactgroup', 'host_display_name', 'host_name',
             'hostgroup', 'hostgroup_alias', 'hostgroup_name',
-            'service', 'service_description', 'service_display_name',
+            'service', 'service_contact', 'service_contactgroup', 'service_description', 'service_display_name',
             'servicegroup'
         );
     }
