@@ -3,6 +3,7 @@
 
 namespace Icinga\Module\Monitoring\Web\Controller;
 
+use Exception;
 use Icinga\Module\Monitoring\Controller;
 use Icinga\Module\Monitoring\Forms\Command\Object\CheckNowCommandForm;
 use Icinga\Module\Monitoring\Forms\Command\Object\DeleteCommentCommandForm;
@@ -46,7 +47,7 @@ abstract class MonitoredObjectController extends Controller
             $this->view->tickets = Hook::first('ticket');
         }
         if (Hook::has('grapher')) {
-            $this->view->grapher = Hook::first('grapher');
+            $this->view->graphers = Hook::all('grapher');
         }
     }
 
@@ -84,10 +85,20 @@ abstract class MonitoredObjectController extends Controller
         $this->view->extensionsHtml = array();
         foreach (Hook::all('Monitoring\DetailviewExtension') as $hook) {
             /** @var DetailviewExtensionHook $hook */
-            $this->view->extensionsHtml[] =
-                '<div class="icinga-module module-' . $this->view->escape($hook->getModule()->getName()) . '">'
-                . $hook->setView($this->view)->getHtmlForObject($this->object)
-                . '</div>';
+
+            try {
+                $html = $hook->setView($this->view)->getHtmlForObject($this->object);
+            } catch (Exception $e) {
+                $html = $this->view->escape($e->getMessage());
+            }
+
+            if ($html) {
+                $module = $this->view->escape($hook->getModule()->getName());
+                $this->view->extensionsHtml[] =
+                    '<div class="icinga-module module-' . $module . '" data-icinga-module="' . $module . '">'
+                    . $html
+                    . '</div>';
+            }
         }
     }
 
@@ -145,7 +156,10 @@ abstract class MonitoredObjectController extends Controller
             );
             $groupName = $this->object->getType() . 'groups';
             $payload[$groupName] = $this->object->$groupName;
-            $this->getResponse()->json()->setSuccessData($payload)->sendResponse();
+            $this->getResponse()->json()
+                ->setSuccessData($payload)
+                ->setAutoSanitize()
+                ->sendResponse();
         }
     }
 
@@ -257,7 +271,11 @@ abstract class MonitoredObjectController extends Controller
     protected function setupQuickActionForms()
     {
         $auth = $this->Auth();
-        if ($auth->hasPermission('monitoring/command/schedule-check')) {
+        if ($auth->hasPermission('monitoring/command/schedule-check')
+            || ($auth->hasPermission('monitoring/command/schedule-check/active-only')
+                && $this->object->active_checks_enabled
+            )
+        ) {
             $this->view->checkNowForm = $checkNowForm = new CheckNowCommandForm();
             $checkNowForm
                 ->setObjects($this->object)
